<?php
/**
 * MySQL Demux
 * by Scott Noyes, 2006
 *
 * Attempts to run some SQL statements through a number of MySQL installations.
 * Intended for comparing version behavior, but could be used to check status
 * of multiple servers, or load several slaves at once, or whatever.
 */

error_reporting(E_ALL);
require_once 'Console/Getopt.php';

$options = parseCommandLine();

$compareVersion = null;
$mysqlOptions = '';
$ini = null;
foreach ($options[0] as $option) {
	switch ($option[0]) {
		case 'v':
		case '--version':
			$compareVersion = $option[1];
			break;
		case 'o':
		case '--option':
			$mysqlOptions = $option[1];
			break;
		case 'i':
		case '--ini':
			$ini = $option[1];
			break;
		case '?':
		case '--help':
		default:
			displayHelp(); exit;
			break;
	}
}

$sql = getSQL($options);

// Run the SQL through each of the versions, storing the results
$results = array();
$servers = getServerList($compareVersion, $ini);
foreach ($servers as $key=>$server) {
	$results[runSQL($server, $sql, $mysqlOptions)][] = $key;
}

displayResults($results);

function displayHelp() {
	echo <<<TEXT
MySQL Demux
Scott Noyes, 2006
Runs SQL through multiple installations of MySQL.
Usage: mysqlDumux.php [OPTIONS] [SQL]
If SQL is not provided, reads from standard input.
Options:
  -?, --help            Display this help and exit.
  -i, --ini=name        Use specified ini file for server list. 
                        Defaults to mysqlDemux.ini in current directory.
  -o, --options=name    Options to pass through to MySQL.
  -v, --version=name    Run only version specified or greater.
TEXT;
}

function displayResults($results) {
	foreach ($results as $result => $versions)
		printf("%s\n%s\n", implode(", ", $versions), $result);
}

function getServerList($compareVersion) {
	if (empty($ini))
		$ini = 'mysqlDemux.ini';

	$servers = parse_ini_file($ini, true);

	foreach ($servers as $server) {
		if (empty($server['basepath']))
			die ('basepath required in .ini');
		if (empty($server['version']))
			die ('version in .ini');
	}

	if ($compareVersion !== NULL) {
		$checkVersion = create_function('$v', 'return ' . $compareVersion  . '<= $v["version"];');
		$servers = array_filter($servers, $checkVersion);
	}

	return $servers;
}

function getSQL($options) {
	if (!empty($options[1]))
		$sql = implode(' ', $options[1]);
	else
		$sql = file_get_contents("php://stdin");

	return $sql;
}

function parseCommandLine() {
	$shortopt = "v:o:i:h";
	$longopt = array('version=', 'option=', 'ini=', 'help');
	$con  = new Console_Getopt;
	$args = $con->readPHPArgv();
	array_shift($args);
	$options = $con->getopt2($args, $shortopt, $longopt);
	if (PEAR::isError($options)) {
		die ("Error in command line: " . $options->getMessage() . "\n");
	} 

	return $options;
}

function runSQL($server, $sql, $mysqlOptions) {
	$descriptorspec = array (
		0 => array("pipe", "r"),
		1 => array("pipe", "w"),
		2 => array("pipe", "w")
	);

	$pipes = array();

	$proc = "\"{$server['basepath']}\\bin\\mysql.exe\" $mysqlOptions";
	foreach ($server as $key=>$value) {
		if ($key != 'basepath' && $key != 'version')
			$proc .= " --{$key}={$value}";
	}

	$process = proc_open(
		$proc,
		$descriptorspec, 
		$pipes);

	if (is_resource($process)) {
		fwrite($pipes[0], $sql);
		fclose($pipes[0]);

		$result = stream_get_contents($pipes[1]);
		fclose($pipes[1]);
		$errors = stream_get_contents($pipes[2]);
		fclose($pipes[2]);

		$return_value = proc_close($process);

		return ($errors . $result);
	}
	else
		return false;
}

?>
